unit uIso2022jpEncoding;

interface

uses
  Windows, SysUtils, Classes;

type
  TIso2022jpEncoding = class(TEncoding)
  strict protected
    function GetByteCount(Chars: PChar; CharCount: Integer): Integer; overload; override;
    function GetBytes(Chars: PChar; CharCount: Integer; Bytes: PByte; ByteCount: Integer): Integer; overload; override;
    function GetCharCount(Bytes: PByte; ByteCount: Integer): Integer; overload; override;
    function GetChars(Bytes: PByte; ByteCount: Integer; Chars: PChar; CharCount: Integer): Integer; overload; override;
  public
    constructor Create;
    function GetMaxByteCount(CharCount: Integer): Integer; override;
    function GetMaxCharCount(ByteCount: Integer): Integer; override;
    function GetPreamble: TBytes; override;
  end;

function ConvertINetUnicodeToMultiByte(lpdwMode: LPDWORD; dwEncoding: DWORD; lpSrcStr: LPCWSTR;
 lpnWideCharCount: PInteger; lpDstStr: LPSTR; lpnMultiCharCount: PInteger): HRESULT; stdcall; external 'MLang.dll';
function ConvertINetMultiByteToUnicode(lpdwMode: LPDWORD; dwSrcEncoding: DWORD; lpSrcStr: LPCSTR;
 lpnMultiCharCount: PInteger; lpDstStr: LPWSTR; lpnWideCharCount: PInteger): HRESULT; stdcall; external 'MLang.dll';
function ConvertINetString(lpdwMode: LPDWORD; dwSrcEncoding: DWORD; dwDstEncoding: DWORD;
 lpSrcStr: LPCSTR; lpnSrcSize: PInteger; lpDstStr: Pointer; lpnDstSize: PInteger):Hresult; stdcall; external 'MLang.dll';

implementation

{ TIso2022jpEncoding }

constructor TIso2022jpEncoding.Create;
begin
  //Constructor
  FMaxCharSize := 2;
  FIsSingleByte := False;
end;

function TIso2022jpEncoding.GetByteCount(Chars: PChar; CharCount: Integer): Integer;
var
  dwMode: DWORD;
  SrcBuf, DstBuf: RawByteString;
  SrcSize, DstSize: Integer;
begin
  //GetByteCount
  Result := 0;
  dwMode := 0;
  SrcSize := 0;
  SetLength(SrcBuf,  0);
  if ConvertINetUnicodeToMultiByte(@dwMode, 932, Chars, @CharCount, PAnsiChar(SrcBuf), @SrcSize) = S_OK then
    begin
      DstSize := Length(SrcBuf) * 3 + 3;
      SetLength(DstBuf, DstSize);
      if ConvertINetString(@dwMode, 932, 50220, PAnsiChar(SrcBuf), @SrcSize, PAnsiChar(DstBuf), @DstSize) = S_OK then
        Result := DstSize;
    end;
end;

function TIso2022jpEncoding.GetBytes(Chars: PChar; CharCount: Integer; Bytes: PByte; ByteCount: Integer): Integer;
var
  dwMode: DWORD;
  SrcBuf, DstBuf: RawByteString;
  SrcSize, DstSize: Integer;
begin
  //GetBytes
  Result := 0;
  dwMode := 0;
  SrcSize := 0;
  SetLength(SrcBuf,  0);
  if ConvertINetUnicodeToMultiByte(@dwMode, 932, Chars, @CharCount, PAnsiChar(SrcBuf), @SrcSize) = S_OK then
    begin
      DstSize := Length(SrcBuf) * 3 + 3;
      SetLength(DstBuf, DstSize);
      if ConvertINetString(@dwMode, 932, 50220, PAnsiChar(SrcBuf), @SrcSize, PAnsiChar(DstBuf), @ByteCount) = S_OK then
        Result := ByteCount;
    end;
end;

function TIso2022jpEncoding.GetCharCount(Bytes: PByte; ByteCount: Integer): Integer;
var
  dwMode: DWORD;
  WS: String;
  SrcBuf: RawByteString;
  SrcSize, DstSize: Integer;
begin
  //GetCharCount
  Result := 0;
  dwMode := 0;
  WS := '';
  SrcSize := ByteCount * 3 + 3;
  DstSize := 0;
  SetLength(SrcBuf, SrcSize);
  if ConvertINetString(@dwMode, 50220, 932, PAnsiChar(Bytes), @ByteCount, PAnsiChar(SrcBuf), @SrcSize) = S_OK then
    if ConvertINetMultiByteToUnicode(@dwMode, 932, PAnsiChar(SrcBuf), @SrcSize, PChar(WS), @DstSize) = S_OK then
      Result := DstSize;
end;

function TIso2022jpEncoding.GetChars(Bytes: PByte; ByteCount: Integer; Chars: PChar; CharCount: Integer): Integer;
var
  dwMode: DWORD;
  SrcBuf: RawByteString;
  SrcSize: Integer;
begin
  //GetChars
  Result := 0;
  dwMode := 0;
  SrcSize := ByteCount * 3 + 3;
  SetLength(SrcBuf, SrcSize);
  if ConvertINetString(@dwMode, 50220, 932, PAnsiChar(Bytes), @ByteCount, PAnsiChar(SrcBuf), @SrcSize) = S_OK then
    if ConvertINetMultiByteToUnicode(@dwMode, 932, PAnsiChar(SrcBuf), @SrcSize, Chars, @CharCount) = S_OK then
      Result := CharCount;
end;

function TIso2022jpEncoding.GetMaxByteCount(CharCount: Integer): Integer;
begin
  //GetMaxByteCount
  Result := (CharCount + 1) * FMaxCharSize;
end;

function TIso2022jpEncoding.GetMaxCharCount(ByteCount: Integer): Integer;
begin
  //GetMaxCharCount
  Result := ByteCount;
end;

function TIso2022jpEncoding.GetPreamble: TBytes;
begin
  //GetPreamble
  SetLength(Result, 0);
end;

end.
